@@ -39,6 +39,7 @@ import input_event
 import keybindings
 import orca_glade
 import orca_gui_prefs
+import orca_prefs
 import orca_state
 import platform
 import settings
@@ -51,7 +52,7 @@ appScript = None
 
 class OrcaSetupGUI(orca_gui_prefs.OrcaSetupGUI):
 
-    def __init__(self, fileName, windowName):
+    def __init__(self, fileName, windowName, prefsDict = None):
         """Initialize the application specific Orca configuration GUI.
 
         Arguments:
@@ -59,7 +60,8 @@ class OrcaSetupGUI(orca_gui_prefs.OrcaSetupGUI):
         - windowName: name of the component to get from the Glade file.
         """
 
-        orca_gui_prefs.OrcaSetupGUI.__init__(self, fileName, windowName)
+        orca_gui_prefs.OrcaSetupGUI.__init__(self, fileName, 
+                                             windowName, prefsDict)
 
         # Initialize variables to None to keep pylint happy.
         #
@@ -92,7 +94,7 @@ class OrcaSetupGUI(orca_gui_prefs.OrcaSetupGUI):
 
         vbox = self.appScript.getAppPreferencesGUI()
         if vbox:
-            label = gtk.Label(orca_state.locusOfFocus.getApplication().name)
+            label = gtk.Label(orca_state.activeScript.app.name)
             self.get_widget("notebook").append_page(vbox, label)
 
     def _createPronunciationTreeView(self, pronunciations=None):
@@ -116,7 +118,7 @@ class OrcaSetupGUI(orca_gui_prefs.OrcaSetupGUI):
         # Adjust the title of the app-specific Orca Preferences dialog to
         # include the name of the application.
         #
-        self.app = orca_state.locusOfFocus.getApplication()
+        self.app = orca_state.activeScript.app
         self.applicationName = self.app.name 
         title = _("Orca Preferences for %s") % self.applicationName
         self.get_widget("orcaSetupWindow").set_title(title)
@@ -292,7 +294,7 @@ def showPreferencesUI():
 
     # The name of the application that currently has focus.
     #
-    applicationName = orca_state.locusOfFocus.getApplication().name
+    applicationName = orca_state.activeScript.app.name
 
     removeGeneralPane = False
     if not orca_state.appOS and not orca_state.orcaOS:
@@ -305,13 +307,21 @@ def showPreferencesUI():
         braille.displayMessage(line)
         speech.speak(line)
 
+        prefsDict = orca_prefs.readPreferences()
         orca_state.prefsGladeFile = os.path.join(platform.prefix,
                                                  platform.datadirname,
                                                  platform.package,
                                                  "glade",
                                                  "orca-setup.glade")
+        orca_state.advancedMag = \
+          orca_gui_prefs.OrcaAdvancedMagGUI(orca_state.prefsGladeFile,
+                                   "orcaMagAdvancedDialog", prefsDict)
+        orca_state.advancedMag.init()
+        orca_state.advancedMagDialog = \
+                           orca_state.advancedMag.getAdvancedMagDialog()
+
         orca_state.appOS = OrcaSetupGUI(orca_state.prefsGladeFile,
-                                        "orcaSetupWindow")
+                                        "orcaSetupWindow", prefsDict)
         removeGeneralPane = True
         orca_state.appOS.init()
     else:
