@@ -19,28 +19,26 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py 2558 2007-07-27 18:48:34Z richb $"
-__version__   = "$Revision: 2558 $"
-__date__      = "$Date: 2007-07-27 14:48:34 -0400 (Fri, 27 Jul 2007) $"
+__id__        = "$Id: gnomespeechfactory.py 3845 2008-04-24 19:15:08Z wwalker $"
+__version__   = "$Revision: 3845 $"
+__date__      = "$Date: 2008-04-24 15:15:08 -0400 (Thu, 24 Apr 2008) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
 log = logging.getLogger("speech")
 
-import os
-
 import gobject
 import Queue
-import string
 import threading
 import time
 
 import bonobo
+import ORBit
 
-import atspi
 import chnames
 import debug
+import orca
 import orca_state
 import punctuation_settings
 import settings
@@ -51,14 +49,14 @@ from acss import ACSS
 
 from orca_i18n import _           # for gettext support
 
-atspi.ORBit.load_typelib('GNOME_Speech')
+ORBit.load_typelib('GNOME_Speech')
 import GNOME.Speech, GNOME__POA.Speech
 
 class _SayAll:
-    def __init__(self, iterator, context, id, progressCallback):
+    def __init__(self, iterator, context, utteranceId, progressCallback):
         self.utteranceIterator   = iterator
         self.currentContext      = context
-        self.idForCurrentContext = id
+        self.idForCurrentContext = utteranceId
         self.progressCallback    = progressCallback
 
 class _Speaker(GNOME__POA.Speech.SpeechCallback):
@@ -70,6 +68,13 @@ class _Speaker(GNOME__POA.Speech.SpeechCallback):
     """
 
     def __init__(self, gnome_speaker):
+
+        # We know what we are doing here, so tell pylint not to flag
+        # the self._this() method call as an error.  The disable-msg is
+        # localized to just this method.
+        #
+        # pylint: disable-msg=E1101
+
         self.gnome_speaker = gnome_speaker
         if settings.enableSpeechCallbacks:
             gnome_speaker.registerSpeechCallback(self._this())
@@ -81,19 +86,19 @@ class _Speaker(GNOME__POA.Speech.SpeechCallback):
     def deregisterCallback(self, callback):
         self.__callbacks.remove(callback)
 
-    def notify(self, type, id, offset):
+    def notify(self, progressType, utteranceId, offset):
         """Called by GNOME Speech when the GNOME Speech driver generates
         a callback.
 
         Arguments:
-        - type:   one of GNOME.Speech.speech_callback_speech_started,
+        - progressType:  one of GNOME.Speech.speech_callback_speech_started,
                          GNOME.Speech.speech_callback_speech_progress,
                          GNOME.Speech.speech_callback_speech_ended
-        - id:     the id of the utterance (returned by say)
-        - offset: the character offset into the utterance (for progress)
+        - utteranceId:   the id of the utterance (returned by say)
+        - offset:        the character offset into the utterance (for progress)
         """
         for callback in self.__callbacks:
-            callback.notify(type, id, offset)
+            callback.notify(progressType, utteranceId, offset)
 
     def say(self, text):
         if isinstance(text, unicode):
@@ -128,6 +133,13 @@ class SpeechServer(speechserver.SpeechServer):
     __activeServers = {}
 
     def __activateDriver(iid):
+
+        # We know what we are doing here, so tell pylint not to flag
+        # the _narrow method call as a warning.  The disable-msg is
+        # localized to just this method.
+        #
+        # pylint: disable-msg=W0212
+
         driver = bonobo.activation.activate_from_id(iid,
                                                     0,
                                                     False)
@@ -265,6 +277,8 @@ class SpeechServer(speechserver.SpeechServer):
         self.__gidleId       = 0
         self.__gidleLock     = threading.Lock()
         self.__lastResetTime = 0
+        self.__lastText      = None
+        self.textCharIndices = []
 
     def __getRate(self, speaker):
         """Gets the voice-independent ACSS rate value of a voice."""
@@ -430,14 +444,25 @@ class SpeechServer(speechserver.SpeechServer):
                 voice = voices[0]
 
         s = self.__driver.createSpeaker(voice)
+
+        # We know what we are doing here, so tell pylint not to flag
+        # the _narrow method call as a warning.  The disable-msg is
+        # localized to just this method.
+        #
+        # pylint: disable-msg=W0212
+
         speaker = _Speaker(s._narrow(GNOME.Speech.Speaker))
 
-        # Turn off punctuation if the speaker allows us to do so.  We
-        # do this because we want to handle spoken punctuation on our
-        # own.
+        # Turn off punctuation.  We do this because we want to handle
+        # spoken punctuation on our own.  Only do so if "punctuation mode" 
+        # is available (see # bug #528797).
         #
         try:
-            speaker.setParameterValue("punctuation mode", 0.0)
+            parameters = speaker.getSupportedParameters()
+            for parameter in parameters:
+                if parameter.name == "punctuation mode":
+                    speaker.setParameterValue("punctuation mode", 0.0)
+                    break
         except:
             pass
 
@@ -475,38 +500,36 @@ class SpeechServer(speechserver.SpeechServer):
 
         # Added in the notify method below.
         #
-        # id     = the id of the utterance we sent to gnome-speech
-        # type   = the type of progress we're getting
-        # offset = character offset into the utterance
+        # thisId   = the id of the utterance we sent to gnome-speech
+        # thisType = the type of progress we're getting
+        # offset   = character offset into the utterance
         #
-        (id, type, offset) = self.__eventQueue.get()
+        (thisId, thisType, offset) = self.__eventQueue.get()
 
         offset = self.__adjustTextCharIndex(offset)
 
         if self.__sayAll:
-            if self.__sayAll.idForCurrentContext == id:
+            if self.__sayAll.idForCurrentContext == thisId:
                 context = self.__sayAll.currentContext
-                if type == GNOME.Speech.speech_callback_speech_started:
+                if thisType == GNOME.Speech.speech_callback_speech_started:
                     context.currentOffset = context.startOffset
                     self.__sayAll.progressCallback(
                         self.__sayAll.currentContext,
                         speechserver.SayAllContext.PROGRESS)
-                elif type == GNOME.Speech.speech_callback_speech_progress:
+                elif thisType == GNOME.Speech.speech_callback_speech_progress:
                     context.currentOffset = context.startOffset + offset
                     self.__sayAll.progressCallback(
                         self.__sayAll.currentContext,
                         speechserver.SayAllContext.PROGRESS)
-                elif type == GNOME.Speech.speech_callback_speech_ended:
+                elif thisType == GNOME.Speech.speech_callback_speech_ended:
                     try:
                         while True:
                             [self.__sayAll.currentContext, acss] = \
                                 self.__sayAll.utteranceIterator.next()
                             utterance = self.__sayAll.currentContext.utterance
-                            debug.println(debug.LEVEL_INFO,
-                                          "SPEECH OUTPUT: '" \
-                                          + utterance \
-                                          + "'")
-                            log.info("sayAll utterance='%s'" % utterance)
+                            logLine = "SPEECH OUTPUT: '" + utterance + "'"
+                            debug.println(debug.LEVEL_INFO, logLine)
+                            log.info(logLine)
                             if utterance and len(utterance)\
                                and not utterance.isspace():
                                 self.__sayAll.idForCurrentContext = \
@@ -530,29 +553,29 @@ class SpeechServer(speechserver.SpeechServer):
 
         return rerun
 
-    def notify(self, type, id, offset):
+    def notify(self, progressType, utteranceId, offset):
         """Called by GNOME Speech when the GNOME Speech driver generates
         a callback.  This is for internal use only.
 
         Arguments:
-        - type:   one of GNOME.Speech.speech_callback_speech_started,
+        - progressType:  one of GNOME.Speech.speech_callback_speech_started,
                          GNOME.Speech.speech_callback_speech_progress,
                          GNOME.Speech.speech_callback_speech_ended
-        - id:     the id of the utterance (returned by say)
-        - offset: the character offset into the utterance (for progress)
+        - utteranceId:   the id of the utterance (returned by say)
+        - offset:        the character offset into the utterance (for progress)
         """
 
-        if type == GNOME.Speech.speech_callback_speech_started:
+        if progressType == GNOME.Speech.speech_callback_speech_started:
             self.__isSpeaking = True
-        elif type == GNOME.Speech.speech_callback_speech_progress:
+        elif progressType == GNOME.Speech.speech_callback_speech_progress:
             self.__isSpeaking = True
-        elif (type == GNOME.Speech.speech_callback_speech_ended) \
+        elif (progressType == GNOME.Speech.speech_callback_speech_ended) \
             and (not self.__sayAll):
             self.__isSpeaking = False
 
         if self.__sayAll:
             self.__gidleLock.acquire()
-            self.__eventQueue.put((id, type, offset))
+            self.__eventQueue.put((utteranceId, progressType, offset))
             if not self.__gidleId:
                 if settings.gilSleepTime:
                     time.sleep(settings.gilSleepTime)
@@ -579,7 +602,6 @@ class SpeechServer(speechserver.SpeechServer):
                 families.append(speechserver.VoiceFamily(props))
         except:
             debug.printException(debug.LEVEL_SEVERE)
-            pass
 
         return families
 
@@ -625,20 +647,20 @@ class SpeechServer(speechserver.SpeechServer):
         """
         self.speak(character, acss)
 
-    def speakUtterances(self, list, acss=None, interrupt=True):
+    def speakUtterances(self, utterances, acss=None, interrupt=True):
         """Speaks the given list of utterances immediately.
 
         Arguments:
-        - list:      list of strings to be spoken
-        - acss:      acss.ACSS instance; if None,
-                     the default voice settings will be used.
-                     Otherwise, the acss settings will be
-                     used to augment/override the default
-                     voice settings.
+        - utterances: list of strings to be spoken
+        - acss:       acss.ACSS instance; if None,
+                      the default voice settings will be used.
+                      Otherwise, the acss settings will be
+                      used to augment/override the default
+                      voice settings.
         - interrupt: if True, stop any speech currently in progress.
         """
         i = 0
-        for text in list:
+        for text in utterances:
             if len(text):
                 self.speak(text, acss, interrupt and (i == 0))
             i += 1
@@ -653,11 +675,13 @@ class SpeechServer(speechserver.SpeechServer):
         Returns the equivalent character index into the original text string.
         """
 
+        equivalentIndex = 0
         for i in range(0, len(self.textCharIndices)):
+            equivalentIndex = i
             if self.textCharIndices[i] >= offset:
                 break
 
-        return i
+        return equivalentIndex
 
     def __addVerbalizedPunctuation(self, oldText):
         """Depending upon the users verbalized punctuation setting,
@@ -700,7 +724,7 @@ class SpeechServer(speechserver.SpeechServer):
             oldText = oldText.replace("\n", "", 1)
 
         # Used to keep a list of character offsets into the new text string,
-        # once it has been converted to use verbalized punctuation, one for 
+        # once it has been converted to use verbalized punctuation, one for
         # each character in the original string.
         #
         self.textCharIndices = []
@@ -716,9 +740,9 @@ class SpeechServer(speechserver.SpeechServer):
                 #
                 isPrev = isNext = isSpecial = False
                 if i > 0:
-                    isPrev = not (oldText[i - 1] in string.whitespace)
+                    isPrev = not oldText[i - 1].isspace()
                 if i < (len(oldText) - 1):
-                    isNext = not (oldText[i + 1] in string.whitespace)
+                    isNext = not oldText[i + 1].isspace()
 
                 # If this is a period and there is a non-space character
                 # on either side of it, then always speak it.
@@ -737,9 +761,9 @@ class SpeechServer(speechserver.SpeechServer):
                 if i == 0:
                     prevCharMatches = True
                 if i > 0:
-                    prevCharMatches = (oldText[i - 1] in string.whitespace)
+                    prevCharMatches = oldText[i - 1].isspace()
                 if i < (len(oldText) - 1):
-                    nextCharMatches = (oldText[i + 1] in string.digits or \
+                    nextCharMatches = (oldText[i + 1].isdigit() or \
                                        oldText[i + 1] in currencySymbols)
 
                 if oldText[i] == "-" and \
@@ -867,9 +891,9 @@ class SpeechServer(speechserver.SpeechServer):
         """
         try:
             [context, acss] = utteranceIterator.next()
-            debug.println(debug.LEVEL_INFO,
-                          "SPEECH OUTPUT: '" + context.utterance + "'")
-            log.info("sayAll utterance='%s'" % context.utterance)
+            logLine = "SPEECH OUTPUT: '" + context.utterance + "'"
+            debug.println(debug.LEVEL_INFO, logLine)
+            log.info(logLine)
             self.__sayAll = _SayAll(utteranceIterator,
                                     context,
                                     self.__speak(context.utterance, acss),
@@ -1040,7 +1064,7 @@ class SpeechServer(speechserver.SpeechServer):
             debug.println(debug.LEVEL_SEVERE,
                           "Something looks wrong with speech.  Aborting.")
             debug.printStack(debug.LEVEL_ALL)
-            os._exit(50)
+            orca.abort(50)
         else:
             self.__lastResetTime = time.time()
 
@@ -1063,4 +1087,3 @@ class SpeechServer(speechserver.SpeechServer):
                 except:
                     debug.printException(debug.LEVEL_SEVERE)
                     self.__driver = None
-                    pass
